/*******************************************************************************
* File Name: EMI_SPIM.h
* Version 2.50
*
* Description:
*  Contains the function prototypes, constants and register definition
*  of the SPI Master Component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_SPIM_EMI_SPIM_H)
#define CY_SPIM_EMI_SPIM_H

#include "cytypes.h"
#include "cyfitter.h"
#include "CyLib.h"

/* Check to see if required defines such as CY_PSOC5A are available */
/* They are defined starting with cy_boot v3.0 */
#if !defined (CY_PSOC5A)
    #error Component SPI_Master_v2_50 requires cy_boot v3.0 or later
#endif /* (CY_PSOC5A) */


/***************************************
*   Conditional Compilation Parameters
***************************************/

#define EMI_SPIM_INTERNAL_CLOCK             (1u)

#if(0u != EMI_SPIM_INTERNAL_CLOCK)
    #include "EMI_SPIM_IntClock.h"
#endif /* (0u != EMI_SPIM_INTERNAL_CLOCK) */

#define EMI_SPIM_MODE                       (1u)
#define EMI_SPIM_DATA_WIDTH                 (8u)
#define EMI_SPIM_MODE_USE_ZERO              (1u)
#define EMI_SPIM_BIDIRECTIONAL_MODE         (0u)

/* Internal interrupt handling */
#define EMI_SPIM_TX_BUFFER_SIZE             (4u)
#define EMI_SPIM_RX_BUFFER_SIZE             (4u)
#define EMI_SPIM_INTERNAL_TX_INT_ENABLED    (0u)
#define EMI_SPIM_INTERNAL_RX_INT_ENABLED    (0u)

#define EMI_SPIM_SINGLE_REG_SIZE            (8u)
#define EMI_SPIM_USE_SECOND_DATAPATH        (EMI_SPIM_DATA_WIDTH > EMI_SPIM_SINGLE_REG_SIZE)

#define EMI_SPIM_FIFO_SIZE                  (4u)
#define EMI_SPIM_TX_SOFTWARE_BUF_ENABLED    ((0u != EMI_SPIM_INTERNAL_TX_INT_ENABLED) && \
                                                     (EMI_SPIM_TX_BUFFER_SIZE > EMI_SPIM_FIFO_SIZE))

#define EMI_SPIM_RX_SOFTWARE_BUF_ENABLED    ((0u != EMI_SPIM_INTERNAL_RX_INT_ENABLED) && \
                                                     (EMI_SPIM_RX_BUFFER_SIZE > EMI_SPIM_FIFO_SIZE))


/***************************************
*        Data Struct Definition
***************************************/

/* Sleep Mode API Support */
typedef struct
{
    uint8 enableState;
    uint8 cntrPeriod;
} EMI_SPIM_BACKUP_STRUCT;


/***************************************
*        Function Prototypes
***************************************/

void  EMI_SPIM_Init(void)                           ;
void  EMI_SPIM_Enable(void)                         ;
void  EMI_SPIM_Start(void)                          ;
void  EMI_SPIM_Stop(void)                           ;

void  EMI_SPIM_EnableTxInt(void)                    ;
void  EMI_SPIM_EnableRxInt(void)                    ;
void  EMI_SPIM_DisableTxInt(void)                   ;
void  EMI_SPIM_DisableRxInt(void)                   ;

void  EMI_SPIM_Sleep(void)                          ;
void  EMI_SPIM_Wakeup(void)                         ;
void  EMI_SPIM_SaveConfig(void)                     ;
void  EMI_SPIM_RestoreConfig(void)                  ;

void  EMI_SPIM_SetTxInterruptMode(uint8 intSrc)     ;
void  EMI_SPIM_SetRxInterruptMode(uint8 intSrc)     ;
uint8 EMI_SPIM_ReadTxStatus(void)                   ;
uint8 EMI_SPIM_ReadRxStatus(void)                   ;
void  EMI_SPIM_WriteTxData(uint8 txData)  \
                                                            ;
uint8 EMI_SPIM_ReadRxData(void) \
                                                            ;
uint8 EMI_SPIM_GetRxBufferSize(void)                ;
uint8 EMI_SPIM_GetTxBufferSize(void)                ;
void  EMI_SPIM_ClearRxBuffer(void)                  ;
void  EMI_SPIM_ClearTxBuffer(void)                  ;
void  EMI_SPIM_ClearFIFO(void)                              ;
void  EMI_SPIM_PutArray(const uint8 buffer[], uint8 byteCount) \
                                                            ;

#if(0u != EMI_SPIM_BIDIRECTIONAL_MODE)
    void  EMI_SPIM_TxEnable(void)                   ;
    void  EMI_SPIM_TxDisable(void)                  ;
#endif /* (0u != EMI_SPIM_BIDIRECTIONAL_MODE) */

CY_ISR_PROTO(EMI_SPIM_TX_ISR);
CY_ISR_PROTO(EMI_SPIM_RX_ISR);


/***************************************
*   Variable with external linkage
***************************************/

extern uint8 EMI_SPIM_initVar;


/***************************************
*           API Constants
***************************************/

#define EMI_SPIM_TX_ISR_NUMBER     ((uint8) (EMI_SPIM_TxInternalInterrupt__INTC_NUMBER))
#define EMI_SPIM_RX_ISR_NUMBER     ((uint8) (EMI_SPIM_RxInternalInterrupt__INTC_NUMBER))

#define EMI_SPIM_TX_ISR_PRIORITY   ((uint8) (EMI_SPIM_TxInternalInterrupt__INTC_PRIOR_NUM))
#define EMI_SPIM_RX_ISR_PRIORITY   ((uint8) (EMI_SPIM_RxInternalInterrupt__INTC_PRIOR_NUM))


/***************************************
*    Initial Parameter Constants
***************************************/

#define EMI_SPIM_INT_ON_SPI_DONE    ((uint8) (0u   << EMI_SPIM_STS_SPI_DONE_SHIFT))
#define EMI_SPIM_INT_ON_TX_EMPTY    ((uint8) (0u   << EMI_SPIM_STS_TX_FIFO_EMPTY_SHIFT))
#define EMI_SPIM_INT_ON_TX_NOT_FULL ((uint8) (0u << \
                                                                           EMI_SPIM_STS_TX_FIFO_NOT_FULL_SHIFT))
#define EMI_SPIM_INT_ON_BYTE_COMP   ((uint8) (0u  << EMI_SPIM_STS_BYTE_COMPLETE_SHIFT))
#define EMI_SPIM_INT_ON_SPI_IDLE    ((uint8) (0u   << EMI_SPIM_STS_SPI_IDLE_SHIFT))

/* Disable TX_NOT_FULL if software buffer is used */
#define EMI_SPIM_INT_ON_TX_NOT_FULL_DEF ((EMI_SPIM_TX_SOFTWARE_BUF_ENABLED) ? \
                                                                        (0u) : (EMI_SPIM_INT_ON_TX_NOT_FULL))

/* TX interrupt mask */
#define EMI_SPIM_TX_INIT_INTERRUPTS_MASK    (EMI_SPIM_INT_ON_SPI_DONE  | \
                                                     EMI_SPIM_INT_ON_TX_EMPTY  | \
                                                     EMI_SPIM_INT_ON_TX_NOT_FULL_DEF | \
                                                     EMI_SPIM_INT_ON_BYTE_COMP | \
                                                     EMI_SPIM_INT_ON_SPI_IDLE)

#define EMI_SPIM_INT_ON_RX_FULL         ((uint8) (0u << \
                                                                          EMI_SPIM_STS_RX_FIFO_FULL_SHIFT))
#define EMI_SPIM_INT_ON_RX_NOT_EMPTY    ((uint8) (0u << \
                                                                          EMI_SPIM_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define EMI_SPIM_INT_ON_RX_OVER         ((uint8) (0u << \
                                                                          EMI_SPIM_STS_RX_FIFO_OVERRUN_SHIFT))

/* RX interrupt mask */
#define EMI_SPIM_RX_INIT_INTERRUPTS_MASK    (EMI_SPIM_INT_ON_RX_FULL      | \
                                                     EMI_SPIM_INT_ON_RX_NOT_EMPTY | \
                                                     EMI_SPIM_INT_ON_RX_OVER)
/* Nubmer of bits to receive/transmit */
#define EMI_SPIM_BITCTR_INIT            (((uint8) (EMI_SPIM_DATA_WIDTH << 1u)) - 1u)


/***************************************
*             Registers
***************************************/
#if(CY_PSOC3 || CY_PSOC5)
    #define EMI_SPIM_TXDATA_REG (* (reg8 *) \
                                                EMI_SPIM_BSPIM_sR8_Dp_u0__F0_REG)
    #define EMI_SPIM_TXDATA_PTR (  (reg8 *) \
                                                EMI_SPIM_BSPIM_sR8_Dp_u0__F0_REG)
    #define EMI_SPIM_RXDATA_REG (* (reg8 *) \
                                                EMI_SPIM_BSPIM_sR8_Dp_u0__F1_REG)
    #define EMI_SPIM_RXDATA_PTR (  (reg8 *) \
                                                EMI_SPIM_BSPIM_sR8_Dp_u0__F1_REG)
#else   /* PSOC4 */
    #if(EMI_SPIM_USE_SECOND_DATAPATH)
        #define EMI_SPIM_TXDATA_REG (* (reg16 *) \
                                          EMI_SPIM_BSPIM_sR8_Dp_u0__16BIT_F0_REG)
        #define EMI_SPIM_TXDATA_PTR (  (reg16 *) \
                                          EMI_SPIM_BSPIM_sR8_Dp_u0__16BIT_F0_REG)
        #define EMI_SPIM_RXDATA_REG (* (reg16 *) \
                                          EMI_SPIM_BSPIM_sR8_Dp_u0__16BIT_F1_REG)
        #define EMI_SPIM_RXDATA_PTR (  (reg16 *) \
                                          EMI_SPIM_BSPIM_sR8_Dp_u0__16BIT_F1_REG)
    #else
        #define EMI_SPIM_TXDATA_REG (* (reg8 *) \
                                                EMI_SPIM_BSPIM_sR8_Dp_u0__F0_REG)
        #define EMI_SPIM_TXDATA_PTR (  (reg8 *) \
                                                EMI_SPIM_BSPIM_sR8_Dp_u0__F0_REG)
        #define EMI_SPIM_RXDATA_REG (* (reg8 *) \
                                                EMI_SPIM_BSPIM_sR8_Dp_u0__F1_REG)
        #define EMI_SPIM_RXDATA_PTR (  (reg8 *) \
                                                EMI_SPIM_BSPIM_sR8_Dp_u0__F1_REG)
    #endif /* (EMI_SPIM_USE_SECOND_DATAPATH) */
#endif     /* (CY_PSOC3 || CY_PSOC5) */

#define EMI_SPIM_AUX_CONTROL_DP0_REG (* (reg8 *) \
                                        EMI_SPIM_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)
#define EMI_SPIM_AUX_CONTROL_DP0_PTR (  (reg8 *) \
                                        EMI_SPIM_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)

#if(EMI_SPIM_USE_SECOND_DATAPATH)
    #define EMI_SPIM_AUX_CONTROL_DP1_REG  (* (reg8 *) \
                                        EMI_SPIM_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
    #define EMI_SPIM_AUX_CONTROL_DP1_PTR  (  (reg8 *) \
                                        EMI_SPIM_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
#endif /* (EMI_SPIM_USE_SECOND_DATAPATH) */

#define EMI_SPIM_COUNTER_PERIOD_REG     (* (reg8 *) EMI_SPIM_BSPIM_BitCounter__PERIOD_REG)
#define EMI_SPIM_COUNTER_PERIOD_PTR     (  (reg8 *) EMI_SPIM_BSPIM_BitCounter__PERIOD_REG)
#define EMI_SPIM_COUNTER_CONTROL_REG    (* (reg8 *) EMI_SPIM_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)
#define EMI_SPIM_COUNTER_CONTROL_PTR    (  (reg8 *) EMI_SPIM_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)

#define EMI_SPIM_TX_STATUS_REG          (* (reg8 *) EMI_SPIM_BSPIM_TxStsReg__STATUS_REG)
#define EMI_SPIM_TX_STATUS_PTR          (  (reg8 *) EMI_SPIM_BSPIM_TxStsReg__STATUS_REG)
#define EMI_SPIM_RX_STATUS_REG          (* (reg8 *) EMI_SPIM_BSPIM_RxStsReg__STATUS_REG)
#define EMI_SPIM_RX_STATUS_PTR          (  (reg8 *) EMI_SPIM_BSPIM_RxStsReg__STATUS_REG)

#define EMI_SPIM_CONTROL_REG            (* (reg8 *) \
                                      EMI_SPIM_BSPIM_BidirMode_CtrlReg__CONTROL_REG)
#define EMI_SPIM_CONTROL_PTR            (  (reg8 *) \
                                      EMI_SPIM_BSPIM_BidirMode_CtrlReg__CONTROL_REG)

#define EMI_SPIM_TX_STATUS_MASK_REG     (* (reg8 *) EMI_SPIM_BSPIM_TxStsReg__MASK_REG)
#define EMI_SPIM_TX_STATUS_MASK_PTR     (  (reg8 *) EMI_SPIM_BSPIM_TxStsReg__MASK_REG)
#define EMI_SPIM_RX_STATUS_MASK_REG     (* (reg8 *) EMI_SPIM_BSPIM_RxStsReg__MASK_REG)
#define EMI_SPIM_RX_STATUS_MASK_PTR     (  (reg8 *) EMI_SPIM_BSPIM_RxStsReg__MASK_REG)

#define EMI_SPIM_TX_STATUS_ACTL_REG     (* (reg8 *) EMI_SPIM_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define EMI_SPIM_TX_STATUS_ACTL_PTR     (  (reg8 *) EMI_SPIM_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define EMI_SPIM_RX_STATUS_ACTL_REG     (* (reg8 *) EMI_SPIM_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)
#define EMI_SPIM_RX_STATUS_ACTL_PTR     (  (reg8 *) EMI_SPIM_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)

#if(EMI_SPIM_USE_SECOND_DATAPATH)
    #define EMI_SPIM_AUX_CONTROLDP1     (EMI_SPIM_AUX_CONTROL_DP1_REG)
#endif /* (EMI_SPIM_USE_SECOND_DATAPATH) */


/***************************************
*       Register Constants
***************************************/

/* Status Register Definitions */
#define EMI_SPIM_STS_SPI_DONE_SHIFT             (0x00u)
#define EMI_SPIM_STS_TX_FIFO_EMPTY_SHIFT        (0x01u)
#define EMI_SPIM_STS_TX_FIFO_NOT_FULL_SHIFT     (0x02u)
#define EMI_SPIM_STS_BYTE_COMPLETE_SHIFT        (0x03u)
#define EMI_SPIM_STS_SPI_IDLE_SHIFT             (0x04u)
#define EMI_SPIM_STS_RX_FIFO_FULL_SHIFT         (0x04u)
#define EMI_SPIM_STS_RX_FIFO_NOT_EMPTY_SHIFT    (0x05u)
#define EMI_SPIM_STS_RX_FIFO_OVERRUN_SHIFT      (0x06u)

#define EMI_SPIM_STS_SPI_DONE           ((uint8) (0x01u << EMI_SPIM_STS_SPI_DONE_SHIFT))
#define EMI_SPIM_STS_TX_FIFO_EMPTY      ((uint8) (0x01u << EMI_SPIM_STS_TX_FIFO_EMPTY_SHIFT))
#define EMI_SPIM_STS_TX_FIFO_NOT_FULL   ((uint8) (0x01u << EMI_SPIM_STS_TX_FIFO_NOT_FULL_SHIFT))
#define EMI_SPIM_STS_BYTE_COMPLETE      ((uint8) (0x01u << EMI_SPIM_STS_BYTE_COMPLETE_SHIFT))
#define EMI_SPIM_STS_SPI_IDLE           ((uint8) (0x01u << EMI_SPIM_STS_SPI_IDLE_SHIFT))
#define EMI_SPIM_STS_RX_FIFO_FULL       ((uint8) (0x01u << EMI_SPIM_STS_RX_FIFO_FULL_SHIFT))
#define EMI_SPIM_STS_RX_FIFO_NOT_EMPTY  ((uint8) (0x01u << EMI_SPIM_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define EMI_SPIM_STS_RX_FIFO_OVERRUN    ((uint8) (0x01u << EMI_SPIM_STS_RX_FIFO_OVERRUN_SHIFT))

/* TX and RX masks for clear on read bits */
#define EMI_SPIM_TX_STS_CLR_ON_RD_BYTES_MASK    (0x09u)
#define EMI_SPIM_RX_STS_CLR_ON_RD_BYTES_MASK    (0x40u)

/* StatusI Register Interrupt Enable Control Bits */
/* As defined by the Register map for the AUX Control Register */
#define EMI_SPIM_INT_ENABLE     (0x10u) /* Enable interrupt from statusi */
#define EMI_SPIM_TX_FIFO_CLR    (0x01u) /* F0 - TX FIFO */
#define EMI_SPIM_RX_FIFO_CLR    (0x02u) /* F1 - RX FIFO */
#define EMI_SPIM_FIFO_CLR       (EMI_SPIM_TX_FIFO_CLR | EMI_SPIM_RX_FIFO_CLR)

/* Bit Counter (7-bit) Control Register Bit Definitions */
/* As defined by the Register map for the AUX Control Register */
#define EMI_SPIM_CNTR_ENABLE    (0x20u) /* Enable CNT7 */

/* Bi-Directional mode control bit */
#define EMI_SPIM_CTRL_TX_SIGNAL_EN  (0x01u)

/* Datapath Auxillary Control Register definitions */
#define EMI_SPIM_AUX_CTRL_FIFO0_CLR         (0x01u)
#define EMI_SPIM_AUX_CTRL_FIFO1_CLR         (0x02u)
#define EMI_SPIM_AUX_CTRL_FIFO0_LVL         (0x04u)
#define EMI_SPIM_AUX_CTRL_FIFO1_LVL         (0x08u)
#define EMI_SPIM_STATUS_ACTL_INT_EN_MASK    (0x10u)

/* Component disabled */
#define EMI_SPIM_DISABLED   (0u)


/***************************************
*       Macros
***************************************/

/* Returns true if componentn enabled */
#define EMI_SPIM_IS_ENABLED (0u != (EMI_SPIM_TX_STATUS_ACTL_REG & EMI_SPIM_INT_ENABLE))

/* Retuns TX status register */
#define EMI_SPIM_GET_STATUS_TX(swTxSts) ( (uint8)(EMI_SPIM_TX_STATUS_REG | \
                                                          ((swTxSts) & EMI_SPIM_TX_STS_CLR_ON_RD_BYTES_MASK)) )
/* Retuns RX status register */
#define EMI_SPIM_GET_STATUS_RX(swRxSts) ( (uint8)(EMI_SPIM_RX_STATUS_REG | \
                                                          ((swRxSts) & EMI_SPIM_RX_STS_CLR_ON_RD_BYTES_MASK)) )


/***************************************
* The following code is DEPRECATED and 
* should not be used in new projects.
***************************************/

#define EMI_SPIM_WriteByte   EMI_SPIM_WriteTxData
#define EMI_SPIM_ReadByte    EMI_SPIM_ReadRxData
void  EMI_SPIM_SetInterruptMode(uint8 intSrc)       ;
uint8 EMI_SPIM_ReadStatus(void)                     ;
void  EMI_SPIM_EnableInt(void)                      ;
void  EMI_SPIM_DisableInt(void)                     ;

#define EMI_SPIM_TXDATA                 (EMI_SPIM_TXDATA_REG)
#define EMI_SPIM_RXDATA                 (EMI_SPIM_RXDATA_REG)
#define EMI_SPIM_AUX_CONTROLDP0         (EMI_SPIM_AUX_CONTROL_DP0_REG)
#define EMI_SPIM_TXBUFFERREAD           (EMI_SPIM_txBufferRead)
#define EMI_SPIM_TXBUFFERWRITE          (EMI_SPIM_txBufferWrite)
#define EMI_SPIM_RXBUFFERREAD           (EMI_SPIM_rxBufferRead)
#define EMI_SPIM_RXBUFFERWRITE          (EMI_SPIM_rxBufferWrite)

#define EMI_SPIM_COUNTER_PERIOD         (EMI_SPIM_COUNTER_PERIOD_REG)
#define EMI_SPIM_COUNTER_CONTROL        (EMI_SPIM_COUNTER_CONTROL_REG)
#define EMI_SPIM_STATUS                 (EMI_SPIM_TX_STATUS_REG)
#define EMI_SPIM_CONTROL                (EMI_SPIM_CONTROL_REG)
#define EMI_SPIM_STATUS_MASK            (EMI_SPIM_TX_STATUS_MASK_REG)
#define EMI_SPIM_STATUS_ACTL            (EMI_SPIM_TX_STATUS_ACTL_REG)

#define EMI_SPIM_INIT_INTERRUPTS_MASK  (EMI_SPIM_INT_ON_SPI_DONE     | \
                                                EMI_SPIM_INT_ON_TX_EMPTY     | \
                                                EMI_SPIM_INT_ON_TX_NOT_FULL_DEF  | \
                                                EMI_SPIM_INT_ON_RX_FULL      | \
                                                EMI_SPIM_INT_ON_RX_NOT_EMPTY | \
                                                EMI_SPIM_INT_ON_RX_OVER      | \
                                                EMI_SPIM_INT_ON_BYTE_COMP)
                                                
#define EMI_SPIM_DataWidth                  (EMI_SPIM_DATA_WIDTH)
#define EMI_SPIM_InternalClockUsed          (EMI_SPIM_INTERNAL_CLOCK)
#define EMI_SPIM_InternalTxInterruptEnabled (EMI_SPIM_INTERNAL_TX_INT_ENABLED)
#define EMI_SPIM_InternalRxInterruptEnabled (EMI_SPIM_INTERNAL_RX_INT_ENABLED)
#define EMI_SPIM_ModeUseZero                (EMI_SPIM_MODE_USE_ZERO)
#define EMI_SPIM_BidirectionalMode          (EMI_SPIM_BIDIRECTIONAL_MODE)
#define EMI_SPIM_Mode                       (EMI_SPIM_MODE)
#define EMI_SPIM_DATAWIDHT                  (EMI_SPIM_DATA_WIDTH)
#define EMI_SPIM_InternalInterruptEnabled   (0u)

#define EMI_SPIM_TXBUFFERSIZE   (EMI_SPIM_TX_BUFFER_SIZE)
#define EMI_SPIM_RXBUFFERSIZE   (EMI_SPIM_RX_BUFFER_SIZE)

#define EMI_SPIM_TXBUFFER       EMI_SPIM_txBuffer
#define EMI_SPIM_RXBUFFER       EMI_SPIM_rxBuffer

#endif /* (CY_SPIM_EMI_SPIM_H) */


/* [] END OF FILE */
