/*******************************************************************************
* File Name: EMI_SPIM_INT.c
* Version 2.50
*
* Description:
*  This file provides all Interrupt Service Routine (ISR) for the SPI Master
*  component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "EMI_SPIM_PVT.h"
#include "cyapicallbacks.h"

/* User code required at start of ISR */
/* `#START EMI_SPIM_ISR_START_DEF` */

/* `#END` */


/*******************************************************************************
* Function Name: EMI_SPIM_TX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for TX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  EMI_SPIM_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  EMI_SPIM_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified when exist data to
*  sending and FIFO Not Full.
*  EMI_SPIM_txBuffer[EMI_SPIM_TX_BUFFER_SIZE] - used to store
*  data to sending.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(EMI_SPIM_TX_ISR)
{
    #if(EMI_SPIM_TX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
    #endif /* (EMI_SPIM_TX_SOFTWARE_BUF_ENABLED) */

    #ifdef EMI_SPIM_TX_ISR_ENTRY_CALLBACK
        EMI_SPIM_TX_ISR_EntryCallback();
    #endif /* EMI_SPIM_TX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START EMI_SPIM_TX_ISR_START` */

    /* `#END` */
    
    #if(EMI_SPIM_TX_SOFTWARE_BUF_ENABLED)
        /* Check if TX data buffer is not empty and there is space in TX FIFO */
        while(EMI_SPIM_txBufferRead != EMI_SPIM_txBufferWrite)
        {
            tmpStatus = EMI_SPIM_GET_STATUS_TX(EMI_SPIM_swStatusTx);
            EMI_SPIM_swStatusTx = tmpStatus;

            if(0u != (EMI_SPIM_swStatusTx & EMI_SPIM_STS_TX_FIFO_NOT_FULL))
            {
                if(0u == EMI_SPIM_txBufferFull)
                {
                   EMI_SPIM_txBufferRead++;

                    if(EMI_SPIM_txBufferRead >= EMI_SPIM_TX_BUFFER_SIZE)
                    {
                        EMI_SPIM_txBufferRead = 0u;
                    }
                }
                else
                {
                    EMI_SPIM_txBufferFull = 0u;
                }

                /* Put data element into the TX FIFO */
                CY_SET_REG8(EMI_SPIM_TXDATA_PTR, 
                                             EMI_SPIM_txBuffer[EMI_SPIM_txBufferRead]);
            }
            else
            {
                break;
            }
        }

        if(EMI_SPIM_txBufferRead == EMI_SPIM_txBufferWrite)
        {
            /* TX Buffer is EMPTY: disable interrupt on TX NOT FULL */
            EMI_SPIM_TX_STATUS_MASK_REG &= ((uint8) ~EMI_SPIM_STS_TX_FIFO_NOT_FULL);
        }

    #endif /* (EMI_SPIM_TX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START EMI_SPIM_TX_ISR_END` */

    /* `#END` */
    
    #ifdef EMI_SPIM_TX_ISR_EXIT_CALLBACK
        EMI_SPIM_TX_ISR_ExitCallback();
    #endif /* EMI_SPIM_TX_ISR_EXIT_CALLBACK */
}


/*******************************************************************************
* Function Name: EMI_SPIM_RX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for RX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  EMI_SPIM_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer modified when FIFO contains
*  new data.
*  EMI_SPIM_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified when overflow occurred.
*  EMI_SPIM_rxBuffer[EMI_SPIM_RX_BUFFER_SIZE] - used to store
*  received data, modified when FIFO contains new data.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(EMI_SPIM_RX_ISR)
{
    #if(EMI_SPIM_RX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
        uint8 rxData;
    #endif /* (EMI_SPIM_RX_SOFTWARE_BUF_ENABLED) */

    #ifdef EMI_SPIM_RX_ISR_ENTRY_CALLBACK
        EMI_SPIM_RX_ISR_EntryCallback();
    #endif /* EMI_SPIM_RX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START EMI_SPIM_RX_ISR_START` */

    /* `#END` */
    
    #if(EMI_SPIM_RX_SOFTWARE_BUF_ENABLED)

        tmpStatus = EMI_SPIM_GET_STATUS_RX(EMI_SPIM_swStatusRx);
        EMI_SPIM_swStatusRx = tmpStatus;

        /* Check if RX data FIFO has some data to be moved into the RX Buffer */
        while(0u != (EMI_SPIM_swStatusRx & EMI_SPIM_STS_RX_FIFO_NOT_EMPTY))
        {
            rxData = CY_GET_REG8(EMI_SPIM_RXDATA_PTR);

            /* Set next pointer. */
            EMI_SPIM_rxBufferWrite++;
            if(EMI_SPIM_rxBufferWrite >= EMI_SPIM_RX_BUFFER_SIZE)
            {
                EMI_SPIM_rxBufferWrite = 0u;
            }

            if(EMI_SPIM_rxBufferWrite == EMI_SPIM_rxBufferRead)
            {
                EMI_SPIM_rxBufferRead++;
                if(EMI_SPIM_rxBufferRead >= EMI_SPIM_RX_BUFFER_SIZE)
                {
                    EMI_SPIM_rxBufferRead = 0u;
                }

                EMI_SPIM_rxBufferFull = 1u;
            }

            /* Move data from the FIFO to the Buffer */
            EMI_SPIM_rxBuffer[EMI_SPIM_rxBufferWrite] = rxData;

            tmpStatus = EMI_SPIM_GET_STATUS_RX(EMI_SPIM_swStatusRx);
            EMI_SPIM_swStatusRx = tmpStatus;
        }

    #endif /* (EMI_SPIM_RX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START EMI_SPIM_RX_ISR_END` */

    /* `#END` */
    
    #ifdef EMI_SPIM_RX_ISR_EXIT_CALLBACK
        EMI_SPIM_RX_ISR_ExitCallback();
    #endif /* EMI_SPIM_RX_ISR_EXIT_CALLBACK */
}

/* [] END OF FILE */
