/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/

#include "wvrSPI.h"
#include "debug.h"

#define CMD_PROGRAM_PAGE	0x02
#define CMD_READ_DATA		0x03
#define CMD_READ_STATUS		0x05
#define CMD_WRITE_ENABLE	0x06
#define CMD_ERASE_DATA		0x20
#define CMD_READ_DEVICE_ID	0x9f

#define _spiExchange spiExchange

// MARK: Private Functions

/*******************************************************************************
 * Function Name: _spiExchange
 *******************************************************************************
 *
 * Summary:
 *  Performs the SPI command
 *
 *******************************************************************************/
void spiExchange(uint8 buffer[], uint32 count) {
	uint32 i;
	uint8 data;

	EMI_SPIM_ClearTxBuffer();
	EMI_SPIM_ClearRxBuffer();

	for (i = 0; i < count; i++) {
		data = buffer[i];
		//DBG_PRINTF ("PUTARRAY %02X\r\n", data);
		EMI_SPIM_PutArray(&data, 1);

		while(!(EMI_SPIM_ReadTxStatus() & EMI_SPIM_STS_SPI_DONE)) { }

		buffer[i] = EMI_SPIM_ReadRxData();
	}
}

/*******************************************************************************
 * Function Name: _spiExchange
 *******************************************************************************
 *
 * Summary:
 *  Performs the SPI command
 *
 *******************************************************************************/
void spiExchangeWithDelay(uint8 buffer[], uint32 count) {
	uint32 i;
	uint8 data;

	EMI_SPIM_ClearTxBuffer();
	EMI_SPIM_ClearRxBuffer();

	for (i = 0; i < count; i++) {
		data = buffer[i];
		//DBG_PRINTF ("PUTARRAY %02X\r\n", data);
		EMI_SPIM_PutArray(&data, 1);

		while(!(EMI_SPIM_ReadTxStatus() & EMI_SPIM_STS_SPI_DONE)) { }

		buffer[i] = EMI_SPIM_ReadRxData();
		
		if (i == 0) {
			CyDelayUs(50);
		} else {
			CyDelayUs(10);
		}
	}
}

/*******************************************************************************
 * Function Name: _spiStart, _spiFinish
 *******************************************************************************
 *
 * Summary:
 *  Functions when start/stop commands
 *
 *******************************************************************************/
void _spiStart()	{ F_SS_Write(0); }
void _spiFinish()	{ F_SS_Write(1); }

/*******************************************************************************
 * Function Name: _buildCommand
 *******************************************************************************
 *
 * Summary:
 *  Melds the command code to a page address for a 4-byte SPI command to
 *	transfer
 *
 *******************************************************************************/
void _buildCommand (uint8 code, uint32 page, uint8 *command) {
	command[0] = code;
	command[1] = (page & 0x00ff0000) >> 16;
	command[2] = (page & 0x0000ff00) >>  8;
	command[3] = (page & 0x000000ff);
}


// MARK: Public Functions

/*******************************************************************************
 * Function Name: spiWriteEnable
 *******************************************************************************
 *
 * Summary:
 *  Enables writing to the device
 *
 *******************************************************************************/
void spiWriteEnable() {
	spiWaitReady();	// Wait until any previous write is done

	uint8 command	= CMD_WRITE_ENABLE;

	_spiStart();
	_spiExchange(&command, 1);
	_spiFinish();

	spiWaitReady();
}

/*******************************************************************************
 * Function Name: spiGetStatus
 *******************************************************************************
 *
 * Summary:
 *  Reads the status from SPI memory and returns it as a byte of data
 *
 *	bit7    SRWD (status register write protect)
 *	bit6    QE (Quad Enable)
 *	bit5    BP3 (level of protected block)
 *	bit4    BP2 (level of protected block)
 *	bit3    BP1 (level of protected block)
 *	bit2    BP0 (level of protected block)
 *	bit1    WEL (write enable latch)    (1 means write enable)
 *	bit0    WIP (write in progress bit) (1 means write is in operation)
 *
 *******************************************************************************/
void spiGetStatus(uint8 *status) {
	uint8 command	= CMD_READ_STATUS;

	_spiStart();
	_spiExchange(&command, 1);
	_spiExchange(status, 1);
	_spiFinish();
}

/*******************************************************************************
 * Function Name: spiWaitReady
 *******************************************************************************
 *
 * Summary:
 *  Makes sure SPI is not in the middle of a write before doing something
 *
 *******************************************************************************/
void spiWaitReady() {
	uint8 status = 0xff;
	while ((status & 0x01) != 0x00) {
		spiGetStatus(&status);
		//DBG_PRINTF ("    Status => %02X\r\n", status);
	}
}

/*******************************************************************************
 * Function Name: spiGetDeviceID
 *******************************************************************************
 *
 * Summary:
 *  Reads the device ID -> returns 3 bytes
 *    - Byte 0 -> Manufacturer
 *    - Byte 1 -> Memory Type
 *    - Byte 2 -> Memory Density
 *
 *******************************************************************************/
void spiGetDeviceID(uint8 *deviceID) {
	spiWaitReady();	// Wait until any previous write is done

	uint8 command	= CMD_READ_DEVICE_ID;

	_spiStart();
	_spiExchange(&command, 1);
	_spiExchange(deviceID, 3);
	_spiFinish();
}

/*******************************************************************************
 * Function Name: spiErasePage
 *******************************************************************************
 *
 * Summary:
 *  Erases a page so it can be written again
 *
 *******************************************************************************/
void spiErasePage(uint32 page) {
	spiWaitReady();		// Wait until any previous write is done
	spiWriteEnable();	// Set the write enable

	uint8 command[4];
	_buildCommand(CMD_ERASE_DATA, page, command);

	_spiStart();
	_spiExchange(command, 4);
	_spiFinish();
}

/*******************************************************************************
 * Function Name: spiWritePage
 *******************************************************************************
 *
 * Summary:
 *  Writes a chunk of data of length 'length' to the page address
 *
 *******************************************************************************/
void spiWritePage(uint32 page, uint8 *data, uint32 length) {
	spiErasePage (page);	// erase the page
	spiWaitReady();			// Wait until the erase is done
	spiWriteEnable();		// Set the write enable

	uint8 command[4];
	_buildCommand(CMD_PROGRAM_PAGE, page, command);

	_spiStart();
	_spiExchange(command, 4);
	_spiExchange(data, length);
	_spiFinish();
}

/*******************************************************************************
 * Function Name: spiReadPage
 *******************************************************************************
 *
 * Summary:
 *  Writes a chunk of data of length 'length' to the page address
 *
 *******************************************************************************/
void spiReadPage(uint32 page, uint8 *data, uint32 length) {
	spiWaitReady();			// Wait until the erase is done

	uint8 command[4];
	_buildCommand(CMD_READ_DATA, page, command);

	_spiStart();
	_spiExchange(command, 4);
	_spiExchange(data, length);
	_spiFinish();
}



/* [] END OF FILE */
